/**
 *  complext tc for base and attribute
 *  @author yiminghe@gmail.com
 */
KISSY.use("base", function(S, Base) {
    describe("complex base/attribute", function() {

        it("can merge property value object from parent class", function() {
            function a() {
                a.superclass.constructor.apply(this, S.makeArray(arguments));
            }

            a.ATTRS = {
                x:{
                    getter:function() {
                        return 1;
                    }
                }
            };

            S.extend(a, Base);


            function b() {
                b.superclass.constructor.apply(this, S.makeArray(arguments));
            }

            b.ATTRS = {
                x:{
                    value:2
                }
            };
            S.extend(b, a);

            var t = new b();

            expect(t.get("x")).toBe(1);

        });


        it("support validator", function() {

            function a() {
                a.superclass.constructor.apply(this, S.makeArray(arguments));
            }

            a.ATTRS = {
                tt:{
                    validator:function(v) {
                        return v > 1;
                    }
                }
            };

            S.extend(a, Base);

            var t = new a();

            expect(t.set("tt", 10)).not.toBe(false);

            expect(t.get("tt")).toBe(10);

            expect(t.set("tt", 0)).toBe(false);

            expect(t.get("tt")).toBe(10);

        });

        it("support sub attribute name", function() {

            function a() {
                a.superclass.constructor.apply(this, S.makeArray(arguments));
            }

            a.ATTRS = {
                tt:{
                    // do not  use this in real world code
                    // forbid changing value in getter
                    getter:function(v) {
                        v.x.y++;
                        return v;
                    },
                    setter:function(v) {
                        v.x.y++;
                        return v;
                    }
                }
            };

            S.extend(a, Base);

            var t = new a({

                tt:{
                    x:{
                        y:1
                    }
                }
            });
            var ret = [],getterVal,setterVal;
            t.on("beforeTtChange", function(e) {
                ret.push(e.prevVal.x.y);
                ret.push(e.newVal.x.y);
            });
            t.on("afterTtChange", function(e) {
                ret.push(e.prevVal.x.y);
                ret.push(e.newVal.x.y);
            });

            // only can when tt is  a object (not custom object newed from custom clz)
            expect(t.get("tt.x.y")).toBe(3);

            t.set("tt.x.y", 3);

            expect(t.get("tt.x.y")).toBe(5);

            expect(ret).toEqual([4,3,4,4]);
        });

        it("should fire *Change once for set({})", function() {
            function a() {
                a.superclass.constructor.apply(this, S.makeArray(arguments));
            }

            S.extend(a, Base);

            var aa = new a({x:1,y:{z:1}}),
                ok = 0,
                afterAttrChange = {};

            aa.on("*Change", function(e) {
                expect(e.newVal).toEqual([11,{z:22}]);
                expect(e.prevVal).toEqual([1,{z:1}]);
                expect(e.attrName).toEqual(["x","y"]);
                expect(e.subAttrName).toEqual(["x","y.z"]);
                ok ++;
            });
            aa.on("afterXChange", function(e) {
                expect(e.attrName).toBe("x");
                expect(e.newVal).toBe(11);
                expect(e.prevVal).toBe(1);
                expect(e.subAttrName).toBe("x");
                afterAttrChange.x = 1;
            });
            aa.on("afterYChange", function(e) {
                expect(e.attrName).toBe("y");
                expect(e.newVal).toEqual({z:22});
                expect(e.prevVal).toEqual({z:1});
                expect(e.subAttrName).toBe("y.z");
                afterAttrChange.y = 1;
            });

            aa.set({
                x:11,
                "y.z":22
            });

            expect(aa.get("x")).toBe(11);
            expect(aa.get("y.z")).toBe(22);
            expect(ok).toBe(1);

            expect(afterAttrChange.x).toBe(1);
            expect(afterAttrChange.y).toBe(1);
        });


    });
});